package DDGTest::Spice::MetaOnly;

use DDG::Spice;

attribution twitter => 'someone';

category 'software';

topics "programming", "sysadmin";

name "mySpice";

description "describes mySpice";

primary_example_queries "primary trigger for mySpice", "another primary trigger for mySpice";

secondary_example_queries "secondary trigger for mySpice", "another secondary trigger for mySpice";

icon_url "/i/mysite.com.ico";

code_url "http://github.com/mySpice";

source "mySpice|Source";

status "enabled";

1;
