package DDGTest::Goodie::MetaOnly;

use DDG::Goodie;

attribution twitter => 'someone';

category 'software';

topics "programming", "sysadmin";

name "myGoodie";

description "describes myGoodie";

primary_example_queries "primary trigger for myGoodie", "another primary trigger for myGoodie";

secondary_example_queries "secondary trigger for myGoodie", "another secondary trigger for myGoodie";

icon_url "/i/mysite.com.ico";

code_url "http://github.com/myGoodie";

source "myGoodie|Source";

status "enabled";

1;
