package DDGTest::Longtail::MetaOnly;

use DDG::Longtail;

attribution twitter => 'someone';

category "software";

topics "programming", "sysadmin";

name "myLongtail";

description "describes myLongtail";

primary_example_queries "primary trigger for myLongtail", "another primary trigger for myLongtail";

secondary_example_queries "secondary trigger for myLongtail", "another secondary trigger for myLongtail";

icon_url "/i/mysite.com.ico";

code_url "http://github.com/myLongtail";

source "myLongtail|Source";

status "enabled";

1;
