package DDGTest::Fathead::MetaOnly;

use DDG::Fathead;

attribution twitter => 'someone';

category 'software';

topics "programming", "sysadmin";

name "myFathead";

description "describes myFathead";

primary_example_queries "primary trigger for myFathead", "another primary trigger for myFathead";

secondary_example_queries "secondary trigger for myFathead", "another secondary trigger for myFathead";

icon_url "/i/mysite.com.ico";

code_url "http://github.com/myFathead";

source "myFathead|Source";

status "enabled";

1;
